@@ -3,19 +3,31 @@ from pybotx.bot.api.exceptions import (
     UnsupportedBotAPIVersionError,
 )
 from pybotx.bot.api.responses.bot_disabled import (
+    BotAPIBotDisabledErrorData,
     BotAPIBotDisabledResponse,
     build_bot_disabled_response,
 )
 from pybotx.bot.api.responses.command_accepted import build_command_accepted_response
+from pybotx.bot.api.responses.unverified_request import (
+    BotAPIUnverifiedRequestErrorData,
+    BotAPIUnverifiedRequestResponse,
+    build_unverified_request_response,
+)
 from pybotx.bot.bot import Bot
 from pybotx.bot.callbacks.callback_repo_proto import CallbackRepoProto
 from pybotx.bot.exceptions import (
     AnswerDestinationLookupError,
     BotShuttingDownError,
     BotXMethodCallbackNotFoundError,
+    RequestHeadersNotProvidedError,
     UnknownBotAccountError,
+    UnverifiedRequestError,
+)
+from pybotx.bot.handler import (
+    IncomingMessageHandlerFunc,
+    Middleware,
+    SyncSmartAppEventHandlerFunc,
 )
-from pybotx.bot.handler import IncomingMessageHandlerFunc, Middleware
 from pybotx.bot.handler_collector import HandlerCollector
 from pybotx.bot.testing import lifespan_wrapper
 from pybotx.client.exceptions.callbacks import (
@@ -40,12 +52,21 @@ from pybotx.client.exceptions.http import (
     InvalidBotXResponsePayloadError,
     InvalidBotXStatusCodeError,
 )
+from pybotx.client.exceptions.message import MessageNotFoundError
 from pybotx.client.exceptions.notifications import (
     BotIsNotChatMemberError,
     FinalRecipientsListEmptyError,
     StealthModeDisabledError,
 )
 from pybotx.client.exceptions.users import UserNotFoundError
+from pybotx.client.smartapps_api.exceptions import SyncSmartAppEventHandlerNotFoundError
+from pybotx.client.smartapps_api.smartapp_manifest import (
+    SmartappManifest,
+    SmartappManifestWebParams,
+)
+from pybotx.client.smartapps_api.sync_smartapp_event import (
+    SyncSmartAppEventResponsePayload,
+)
 from pybotx.client.stickers_api.exceptions import (
     InvalidEmojiError,
     InvalidImageError,
@@ -61,6 +82,7 @@ from pybotx.models.attachments import (
     OutgoingAttachment,
 )
 from pybotx.models.bot_account import BotAccount, BotAccountWithSecret
+from pybotx.models.bot_catalog import BotsListItem
 from pybotx.models.bot_sender import BotSender
 from pybotx.models.chats import Chat, ChatInfo, ChatInfoMember, ChatListItem
 from pybotx.models.enums import (
@@ -68,6 +90,7 @@ from pybotx.models.enums import (
     ChatTypes,
     ClientPlatforms,
     MentionTypes,
+    SmartappManifestWebLayoutChoices,
     SyncSourceTypes,
     UserKinds,
 )
@@ -108,6 +131,7 @@ from pybotx.models.system_events.chat_created import ChatCreatedEvent, ChatCreat
 from pybotx.models.system_events.cts_login import CTSLoginEvent
 from pybotx.models.system_events.cts_logout import CTSLogoutEvent
 from pybotx.models.system_events.deleted_from_chat import DeletedFromChatEvent
+from pybotx.models.system_events.event_edit import EventEdit
 from pybotx.models.system_events.internal_bot_notification import (
     InternalBotNotificationEvent,
 )
@@ -118,14 +142,17 @@ from pybotx.models.users import UserFromCSV, UserFromSearch
 __all__ = (
     "AddedToChatEvent",
     "AnswerDestinationLookupError",
-    "AttachmentTypes",
     "AttachmentDocument",
     "AttachmentImage",
-    "AttachmentVoice",
+    "AttachmentTypes",
     "AttachmentVideo",
+    "AttachmentVoice",
     "Bot",
+    "BotAPIBotDisabledErrorData",
     "BotAPIBotDisabledResponse",
     "BotAPIMethodFailedCallback",
+    "BotAPIUnverifiedRequestErrorData",
+    "BotAPIUnverifiedRequestResponse",
     "BotAccount",
     "BotAccountWithSecret",
     "BotIsNotChatMemberError",
@@ -134,6 +161,7 @@ __all__ = (
     "BotShuttingDownError",
     "BotXMethodCallbackNotFoundError",
     "BotXMethodFailedCallbackReceivedError",
+    "BotsListItem",
     "BubbleMarkup",
     "Button",
     "ButtonRow",
@@ -157,6 +185,7 @@ __all__ = (
     "DeletedFromChatEvent",
     "Document",
     "EditMessage",
+    "EventEdit",
     "EventNotFoundError",
     "File",
     "FileDeletedError",
@@ -185,6 +214,7 @@ __all__ = (
     "MentionList",
     "MentionTypes",
     "MentionUser",
+    "MessageNotFoundError",
     "MessageStatus",
     "Middleware",
     "OutgoingAttachment",
@@ -193,18 +223,25 @@ __all__ = (
     "RateLimitReachedError",
     "Reply",
     "ReplyMessage",
+    "RequestHeadersNotProvidedError",
     "SmartApp",
     "SmartAppEvent",
-    "SmartAppEvent",
+    "SyncSmartAppEventResponsePayload",
+    "SmartappManifest",
+    "SmartappManifestWebLayoutChoices",
+    "SmartappManifestWebParams",
     "StatusRecipient",
     "StealthModeDisabledError",
     "Sticker",
     "StickerPack",
     "StickerPackOrStickerNotFoundError",
+    "SyncSmartAppEventHandlerFunc",
+    "SyncSmartAppEventHandlerNotFoundError",
     "SyncSourceTypes",
     "UnknownBotAccountError",
     "UnknownSystemEventError",
     "UnsupportedBotAPIVersionError",
+    "UnverifiedRequestError",
     "UserDevice",
     "UserFromCSV",
     "UserFromSearch",
@@ -215,6 +252,7 @@ __all__ = (
     "Voice",
     "build_bot_disabled_response",
     "build_command_accepted_response",
+    "build_unverified_request_response",
     "lifespan_wrapper",
 )
 
