@@ -100,6 +100,7 @@ class BotAPISystemEventTypes(StrEnum):
     INTERNAL_BOT_NOTIFICATION = "system:internal_bot_notification"
     LEFT_FROM_CHAT = "system:left_from_chat"
     SMARTAPP_EVENT = "system:smartapp_event"
+    EVENT_EDIT = "system:event_edit"
 
 
 class BotAPIClientPlatforms(Enum):
@@ -150,6 +151,12 @@ class APISyncSourceTypes(Enum):
     BOTX = "botx"
 
 
+class SmartappManifestWebLayoutChoices(StrEnum):
+    minimal = "minimal"
+    half = "half"
+    full = "full"
+
+
 def convert_client_platform_to_domain(
     client_platform: BotAPIClientPlatforms,
 ) -> ClientPlatforms:
